import tensorflow as tf


def dense_sparse_matmul(denV: tf.Tensor, spW: tf.SparseTensor) -> tf.Tensor:
    """Multiply row vector with sparse matrix

    Parameters:
        denV (tf.Tensor): Dense 1xN row vector.
        spW (tf.SparseTensor): Sparse NxM matrix.

    Returns:
        tf.Tensor: Dense 1xM row vector

    Motivation:
        TF only supports multiplying a sparse matrix with a column vector.
        By transposing, we can use `tf.sparse.sparse_dense_matmul` to
        multiply a row vector with a sparse matrix. The later multiplication
        is commonly used in context of neural networks in TF/Keras.
    """
    # reshape to list of row vectors if neccessary
    if denV.shape.ndims == 1:
        denV = tf.reshape(denV, (1, -1))
    # transpose -> multiply -> transpose back
    return tf.transpose(tf.sparse.sparse_dense_matmul(
        tf.sparse.transpose(spW), tf.transpose(denV)))



import tensorflow as tf
from keras_dense_sparse_matmul import dense_sparse_matmul


class Test_dense_sparse_matmul(tf.test.TestCase):
    def test1(self):
        h = tf.constant([1., 2., 3.])
        W = tf.sparse.SparseTensor(
            indices=([0, 1], [1, 1], [1, 2], [2, 0], [2, 2]),
            values=[1., 2., 3., 4., 5.],
            dense_shape=(3, 3))
        net = dense_sparse_matmul(h, W)
        self.assertAllEqual(net, [[12., 5., 21.]])


if __name__ == "__main__":
    tf.test.main()



from setuptools import setup
import os


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(name='keras-dense-sparse-matmul',
      version='0.2.0',
      description='Multiply row vector with sparse matrix in tensorflow',
      # long_description=read('README.md'),
      # long_description_content_type='text/markdown',
      url='http://github.com/ulf1/keras-dense-sparse-matmul',
      author='Ulf Hamster',
      author_email='554c46@gmail.com',
      license='Apache License 2.0',
      packages=['keras_dense_sparse_matmul'],
      install_requires=[
          'setuptools>=40.0.0',
          'tensorflow==2.*',
          'numpy>=1.18.*'],
      python_requires='>=3.6',
      zip_safe=True)
